local assets = require('assets')
local const = require('const')
local GameState = require('GameState')
local Animation = require('Animation')

local mt = {}
mt.__index = mt

function mt:update(dt)
  self.touches_hero = GameState.getCurrent().world:check(self, 'is_hero')
  self.current_anim:update(dt)
end

function mt:draw()
  assets.qdraw(self.current_anim:getFrame(), self.x + 16, self.y, 0, -1, 1)
  if self.touches_hero then
    love.graphics.printf('Press E To Continue', self.x-100, self.y - 32, self.w + 200, 'center')
  end
end

function mt:setAnim(name)
  self.current_anim = self.anims[name]
end

function mt:onHeroAction()
  GameState.getCurrent():trigger('door:open')
end

return {
  new = function(x, y, game_state)
    d = setmetatable({
      is_door = true,
      is_actable = true,
      x = x,
      y = y,
      w = const.tilesize,
      h = const.tilesize,
      anims = {
        idle = Animation.new(1, 4, 0.8),
      }
    }, mt)
	
	d:setAnim('idle')
	return d
  end
}
