local assets = require('assets')
local const = require('const')
local GameState = require('GameState')
local Animation = require('Animation')

local mt = {}
mt.__index = mt

function mt:draw()
  if self.is_dead then
	self:setAnim('death')
  else
    if self.dir == -1 then
      assets.qdraw(self.current_anim:getFrame(), self.x + 16, self.y, 0, -1, 1)
    else
      assets.qdraw(self.current_anim:getFrame(), self.x, self.y)
    end
  end
end

function mt:setAnim(name)
  self.current_anim = self.anims[name]
end

function mt:update(dt)
  -- The enemy just moves in it's current direction.
  GameState.getCurrent().world:move(self, self.x+self.dir*self.speed*dt, self.y, 'is_solid')
  
  self.current_anim:update(dt)

  local is_obstacle_ahead = GameState.getCurrent().world:check({
    x = self.x + self.w/2 + self.w*2/3  * self.dir,
    y = self.y + self.h/2,
    w = 2,
    h = 2
  }, 'is_solid')

  local is_floor_ahead = GameState.getCurrent().world:check({
    x = self.x + self.w/2 + self.w*2/3  * self.dir,
    y = self.y + self.h + 1,
    w = 2,
    h = 2
  }, 'all')

  if is_obstacle_ahead or not is_floor_ahead then
    self:turnBack()
  end
end

-- Similarly to toggling boolean with not you can toggle 1/-1 with * -1.
function mt:turnBack()
  self.dir = self.dir * -1
end

function mt:onTouch(other)
  if other.is_hero then
    GameState.getCurrent():trigger('hero:kill', self, other)
  end
end

-- The enemy "is_stompable", and when the hero jumps on it's head this function is called.
function mt:onStomp()
  self.is_touchable = false -- this enemy no longer will be able to touch (and harm) the hero
  self.is_stompable = false -- once stomped this enemy cannot be stomped again.
  self.is_dead = true -- a dead enemy is not updated and it's drawn differently
end

return {
  new = function(x, y)
    e = setmetatable({
      is_touchable = true,
      is_stompable = true,
      x = x,
      y = y,
      w = const.tilesize,
      h = const.tilesize,
      dir = 1,
      speed = 60,
      anims = {
        run = Animation.new(33, 4, 0.7),
        death = Animation.new(33, 4, 0.7),
      },
    }, mt)
	
	e:setAnim('run')
	return e
  end
}
