local assets = require('assets')
local GameState = require('GameState')
local Animation = require('Animation')

local GRAVITY = 9
local JUMP_SPEED = -3.75
local Jumps = 1

local mt = {}
mt.__index = mt

_G.hx = 0
_G.hy = 0

function mt:update(dt)  
  if self.is_disabled then return end
  
  _G.hx = self.x
  _G.hy = self.y
  
  self:setAnim('idle')

  if love.keyboard.isDown('space') and not self.was_jump_pressed_in_previous_frame and Jumps <= 1 then
    self:setAnim('jump')
    self.vy = JUMP_SPEED
	Jumps = Jumps + 1
  end

  if self:isGrounded() then
	Jumps = 1
  end

  if not self:isGrounded() then
	self:setAnim("jump")
  end

  local dx = 0
  if love.keyboard.isDown('left') then
    self:setAnim('run')
    self.last_direction = -1
    dx = dx - dt * self.speed
  end
  if love.keyboard.isDown('right') then
    self:setAnim('run')
    self.last_direction = 1
    dx = dx + dt * self.speed
  end
  GameState.getCurrent().world:move(self, self.x + dx, self.y + self.vy, 'is_solid')

  self.vy = math.min(self.vy + GRAVITY * dt, 12)
  if self:isGrounded() then self.vy = 0 end

  if love.keyboard.isDown('e') and not self.was_action_pressed_in_previous_frame then
    GameState.getCurrent():trigger('hero:action', self)
  end

  if self.vy > 0 then -- the hero can stomp on something only when it's falling
    local stompables = GameState.getCurrent().world:find({
      x = self.x, y = self.y + self.h + 1,
      w = self.w, h = 1,
    }, 'is_stompable')
    for i, stompable in ipairs(stompables) do
      -- Along with a long platformer tradition whenever you stomp on en enemy you bounce up
      self.vy = JUMP_SPEED / 2
      stompable:onStomp()
    end
  end

  local touchables = GameState.getCurrent().world:find(self, 'is_touchable')
  for _, touchable in ipairs(touchables) do
    touchable:onTouch(self)
  end

  self.current_anim:update(dt)

  self.was_jump_pressed_in_previous_frame = love.keyboard.isDown('space')
  self.was_action_pressed_in_previous_frame = love.keyboard.isDown('e')
end

function mt:draw()  
  if self.last_direction == -1 then
	assets.qdraw(self.current_anim:getFrame(), self.x + 16 - 3, self.y, 0, -1, 1)
  else
	assets.qdraw(self.current_anim:getFrame(), self.x- 3, self.y)
  end
end

function mt:setAnim(name)
  self.current_anim = self.anims[name]
end

function mt:isGrounded()
  return GameState.getCurrent().world:check({ x = self.x, y = self.y+self.h, w = self.w, h = 1 }, 'is_solid')
end

return {
  new = function(x, y)
    h = setmetatable({
      is_hero = true,
      x = x, y = y,
      w = 10, h = 16,
      vy = 0,
      speed = 80,
      anims = {
        idle = Animation.new(21, 4, 1.4),
        run = Animation.new(25, 4, 0.5),
        jump = Animation.new(29, 4, 0.5)
      },
      z_index = 10
    }, mt)

    h:setAnim('idle')
    return h
	
  end
}
