local assets = require('assets32')
local GameState = require('GameState')
local Animation = require('Animation')

local mt = {}
mt.__index = mt

function mt:draw()
  if self.dir == -1 then
    assets.qdraw(self.current_anim:getFrame(), self.x + 32, self.y, 0, -1, 1)
  end
end

function mt:setAnim(name)
  self.current_anim = self.anims[name]
end

function mt:update(dt)
  -- The enemy just moves in it's current direction.
  GameState.getCurrent().world:move(self, self.x, self.y+self.dir*self.speed*dt, 'is_solid')
  
  self.current_anim:update(dt)

  local is_obstacle_ahead = GameState.getCurrent().world:check({
    x = self.x + self.h/2,
    y = self.y + self.w/2 + self.w*2/3  * self.dir,
    w = 2,
    h = 2
  }, 'is_solid')

  local is_floor_ahead = GameState.getCurrent().world:check({
    x = self.y + self.h + 1,
    y = self.x + self.w/2 + self.w*2/3  * self.dir,
    w = 2,
    h = 2
  }, 'all')

  if is_obstacle_ahead then
    self:turnBack()
  end
end

-- Similarly to toggling boolean with not you can toggle 1/-1 with * -1.
function mt:turnBack()
  self.dir = self.dir * -1
end

function mt:onTouch(other)
  if other.is_hero then
    GameState.getCurrent():trigger('hero:kill', self, other)
  end
end

function mt:onStomp()
end

return {
  new = function(x, y)
    s = setmetatable({
      is_touchable = true,
      is_stompable = true,
      x = x,
      y = y,
      w = 32,
      h = 32,
      dir = 1,
      speed = 60,
      anims = {
        idle = Animation.new(1, 2, 0.1),
      }
    }, mt)
	
	s:setAnim('idle')
	return s
  end
}
