local GameState = require('GameState')
local Hero = require('Hero')
local Coin = require('Coin')

local Camera = require 'lib.Camera'
local moonshine = require 'shaders'

_G.points = 0
_G.deaths = -1

_G.hx = 0
_G.hy = 0

_G.paused = false
_G.control = false

function love.load()
  camera = Camera()
  
  love.window.setTitle("LOST")
  love.window.setFullscreen(true, "desktop")

  love.graphics.setBackgroundColor(0.1, 0.1, 0.1)  
  GameState.setCurrent('Play', 1)
  
  bgSound = love.audio.newSource("Bg.mp3", "stream")
  love.audio.setVolume(1.5)
  bgSound:setLooping(true)
  
  effect = moonshine(moonshine.effects.fog)
                    .chain(moonshine.effects.vignette)
  effect.fog.fog_color = {0.05,0.05,0.05}
end


function love.update(dt)
  if not paused then
    if not control then
      bgSound:play()
      if love.keyboard.isDown('return') then
        playing = true
      end

      if Coin.points then
      _G.points = Coin.points
      end
      
      if Hero.hx then
      _G.hx = Hero.hx
      end
      
      if Hero.hy then
      _G.hy = Hero.hy
      end

      GameState.getCurrent():update(dt)
      GameState.update()
      
      camera:update(dt)
      camera:follow(_G.hx * 3.5, _G.hy * 3.5)
    end
  end

  function love.keypressed(key, unicode)
		if key == 'escape' then
       _G.paused = not _G.paused
    end
    if key == 'q' then
      _G.control = not _G.control
    end
	end
end



function love.draw()
  effect(function()
    love.graphics.rectangle("fill", love.graphics.getWidth(), love.graphics.getHeight(), 200,200)
  end)
  
  camera:attach()
  love.graphics.setDefaultFilter('nearest', 'nearest')
  love.graphics.scale(3.5, 3.5)
  
  GameState.getCurrent():draw()
  camera:detach()

  if playing and not paused then
    love.graphics.setColor(1,1,1,1)
    love.graphics.printf("Points: " .. _G.points, 15, 15, 1200, "left", 0, 2.5, 2.5)
    love.graphics.printf("Deaths: " .. _G.deaths, 15, love.graphics.getHeight() - 55, 1200, "left", 0, 2.5, 2.5)
    love.graphics.setColor(1,1,1,1)
  end

  if paused then
    love.graphics.setColor(0,0,0,0.75)
    love.graphics.rectangle("fill",0,0,love.graphics.getWidth(),love.graphics.getHeight())
    love.graphics.setColor(1,1,1,1)
    love.graphics.printf("          Paused\n\nPress Esc To Unpause", love.graphics.getWidth() / 2 - 240, love.graphics.getHeight() / 2 - 130, 1200, "left", 0, 4, 4)
  end

  if playing and control then
    love.graphics.setColor(0,0,0,0.75)
    love.graphics.rectangle("fill",0,0,love.graphics.getWidth(),love.graphics.getHeight())
    love.graphics.setColor(1,1,1,1)
    love.graphics.printf("Controls:\nUSE ARROWS TO MOVE\n\nPRESS SPACE TO JUMP\n\nInstructions:\nPRESS SPACE TWICE TO DOUBLE JUMP\n\nTHE ACTIVATED(SOLID) FLOOR WILL BE BLACK\n\nSWITCH THE LEVER TO ACTIVATE THE FLOOR", -1705, love.graphics.getHeight() / 2 - 330, 1200, "center", 0, 4, 4)
      --a tips
    --love.graphics.printf("", 35, 155, 1200, "left", 0, 2.5, 2.5)
  end

end

return {
  new = function(x, y)
    local e = setmetatable({}, mt)
    return e
  end,
  getposx = function()
    return x
  end
}
